InviteOMatic = LibStub("AceAddon-3.0"):NewAddon("InviteOMatic", "AceConsole-3.0", "AceEvent-3.0")

local options = {
    name = "InviteOMatic",
    handler = InviteOMatic,
    type = 'group',
    args = {
		trigger = {
            type = "input",
            name = "Invite trigger",
            desc = "The trigger keyword to invite people with",
            usage = "<Your invite trigger>",
            get = "GetTrigger",
            set = "SetTrigger",
        },
		convert = {
            type = "input",
            name = "Raid trigger",
            desc = "The trigger keyword to convert party to raid",
            usage = "<Your convert trigger>",
            get = "GetConvertTrigger",
            set = "SetConvertTrigger",
        },
		enabled = {
            type = "toggle",
            name = "Invite triggering",
            desc = "If enabled InviteOMatic will listen for the invite trigger in whispers",
            get = "IsTriggeringEnabled",
            set = "ToggleTriggeringEnabled",
        },
		raidEnable = {
            type = "toggle",
            name = "Raid converting",
            desc = "If enabled InviteOMatic convert party to raid on raid trigger",
            get = "IsConvertTriggeringEnabled",
            set = "ToggleConvertTriggeringEnabled",
        },
		debug = {
            type = "toggle",
            name = "Debug printing",
            desc = "Will print out debug messages",
            get = "IsDebugEnabled",
            set = "ToggleDebugEnabled",
        },
		announce = {
            type = "toggle",
            name = "Invite announcement",
            desc = "If a tries to invite b via !invite b, then b gets a whisper stating who wanted him invited",
            get = "IsAnnounceEnabled",
            set = "ToggleAnnounceEnabled",
        },
		reportdecline = {
            type = "toggle",
            name = "Decline reporting",
            desc = "If a invites b via !invite b and b declines, a will get a whisper stating so",
            get = "IsReportDeclineEnabled",
            set = "ToggleReportDeclineEnabled",
        },
    },
}

local defaults = {
    profile = {
        inviteTrigger = "!invite",
		convertTrigger = "!raid",
		enabled = true,
		raidEnable = true,
		debug = false,
		announce = true,
		reportDecline = true,
		reportAlreadyGrouped = true,
    },
}

function InviteOMatic:OnInitialize()
LibStub("AceConfig-3.0"):RegisterOptionsTable("InviteOMatic", options)
    self.db = LibStub("AceDB-3.0"):New("InviteOMaticDB", defaults, "Default")

	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("InviteOMatic", "InviteOMatic")
    self:RegisterChatCommand("iom", "ChatCommand")
    self:RegisterChatCommand("inviteomatic", "ChatCommand")
	
	InviteOMatic.lastAuthor = "";
	InviteOMatic.lastInviteAttempt = "";
end

function InviteOMatic:OnEnable()
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_BN_WHISPER")
end

function InviteOMatic:OnDisable()
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self:UnregisterEvent("CHAT_MSG_BN_WHISPER")
end

function InviteOMatic:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(InviteOMatic, "iom", "InviteOMatic", input)
    end
end

function InviteOMatic:GetConvertTrigger(info)
	return self.db.profile.convertTrigger
end

function InviteOMatic:SetConvertTrigger(info, newConvertValue)
	self.db.profile.convertTrigger = newConvertValue
end

function InviteOMatic:GetTrigger(info)
	return self.db.profile.inviteTrigger
end

function InviteOMatic:SetTrigger(info, newTriggerValue)
	self.db.profile.inviteTrigger = newTriggerValue
end

function InviteOMatic:CHAT_MSG_WHISPER(event, msg, author)
	self:handleChatMsgWhisper(msg, author, false);
end

function InviteOMatic:CHAT_MSG_BN_WHISPER(event, msg, bnAuthor, a, b, c, d, e, f, g, h, i, j, presenceID)
	a, toon, game, realm = BNGetToonInfo(presenceID);
	self:handleChatMsgWhisper(msg, toon.."-"..realm, true);
end

function InviteOMatic:handleChatMsgWhisper(msg, author, bnFriend)
	self:log("["..author.."] whispered you ["..msg.."]");

	local lowerCaseMsg = string.lower(msg)
	
	if( self.db.profile.enabled ) then
		self:log("Triggering is enabled")
		
		local lowerCaseTrigger = string.lower(self.db.profile.inviteTrigger)
		
		local match = string.match(lowerCaseMsg, ".*"..lowerCaseTrigger..".*")
		local anotherMatch = string.match(lowerCaseMsg, ".*"..lowerCaseTrigger.." ".."(%w+)")
		
		if( match ~= nil ) then
			if( anotherMatch ~= nil ) then
				self:log("AnotherMatch: ["..anotherMatch.."]")
			
				self.lastAuthor = author
				self.lastInviteAttempt = anotherMatch
			
				self:RegisterEvent("CHAT_MSG_SYSTEM")
			
				self:invite(anotherMatch)
			else
				self:invite(author)
			end
		end
		
	end
	
	if( self.db.profile.raidEnable ) then
		self:log("Raid convert triggering is enabled")
		
		local lowerCaseConvertTrigger = string.lower(self.db.profile.convertTrigger)
		local convertMatch = string.match(lowerCaseMsg, ".*"..lowerCaseConvertTrigger..".*")
		
		if( convertMatch ~= nil) then
			if( not IsInRaid() ) then
				self:log("Converting party to raid")
				ConvertToRaid()
			else
				self:log("Already in a raid");
			end
		end
	end
end

function InviteOMatic:CHAT_MSG_SYSTEM(event, msg, author)
	msg = string.lower(msg)
	local otherPlayer = string.upper(string.sub(self.lastInviteAttempt,1,1)) .. string.sub(self.lastInviteAttempt,2)
	
	local pattern = "cannot find player '"..self.lastInviteAttempt.."'"
	local pattern2 = "you have invited "..self.lastInviteAttempt.." to join your group"
	local pattern3 = ""..self.lastInviteAttempt.." joins the party"
	local pattern4 = ""..self.lastInviteAttempt.." declines your group invitation"
	local pattern5 = ""..self.lastInviteAttempt.." is already in a group"
	local pattern6 = ".*"..self.lastInviteAttempt..".* has joined the raid group"
	
	local match = string.match(msg, pattern)
	local match2 = string.match(msg, pattern2)
	local match3 = string.match(msg, pattern3)
	local match4 = string.match(msg, pattern4)
	local match5 = string.match(msg, pattern5)
	local match6 = string.match(msg, pattern6)
	
	if( match ~= nil ) then
		self:log("Invite attempt of other player ["..otherPlayer.."] failed (Player dosent exist/isnt online)")

		self:UnregisterEvent("CHAT_MSG_SYSTEM")
		
		self:invite(self.lastAuthor)
		
		self.lastAuthor = ""
		self.lastInviteAttempt = ""
		
	elseif( match2 ~= nil ) then
		self:log(otherPlayer.." is remote invited")
		
		if( self.db.profile.announce ) then
			-- Send a nice message stating who and why you were invited

			SendChatMessage("[InviteOMatic] "..self.lastAuthor.." wants you invited to my group" ,"WHISPER" ,nil ,self.lastInviteAttempt);
		end
		
	elseif( match3 ~= nil ) then
		self:log(otherPlayer.." joined the party")
		
		self:UnregisterEvent("CHAT_MSG_SYSTEM")
		
		self.lastAuthor = ""
		self.lastInviteAttempt = ""
	
	elseif( match4 ~= nil ) then
		self:log(otherPlayer.." declined the invite")
		
		if( self.db.profile.reportDecline ) then
			--Send the invitee a report that his invite was declined

			SendChatMessage("[InviteOMatic] "..otherPlayer.." declined the invite" ,"WHISPER" ,nil ,self.lastAuthor);
		end
		self:UnregisterEvent("CHAT_MSG_SYSTEM")
		
		self.lastAuthor = ""
		self.lastInviteAttempt = ""
	
	elseif( match5 ~= nil ) then
		self:log(otherPlayer.." is already grouped")
		
		if( self.db.profile.reportAlreadyGrouped ) then
			--Send a message stating that player was in a group

			SendChatMessage("[InviteOMatic] "..otherPlayer.." is already in a group" ,"WHISPER" ,nil ,self.lastAuthor);
		end

		self:UnregisterEvent("CHAT_MSG_SYSTEM")
		
		self.lastAuthor = ""
		self.lastInviteAttempt = ""
	
	elseif( match6 ~= nil ) then
		self:log(otherPlayer.." joined the raid group")
		
		self:UnregisterEvent("CHAT_MSG_SYSTEM")
		
		self.lastAuthor = ""
		self.lastInviteAttempt = ""
	end
	
end

function InviteOMatic:invite(player)
	if( self.db.profile.enabled ) then
		self:log("Inviting ["..player.."]")
		InviteUnit(player)
	end
end

function InviteOMatic:log(msg)
	if( self.db.profile.debug ) then
		self:Print(msg)
	end
end

function InviteOMatic:IsTriggeringEnabled(info)
	return self.db.profile.enabled
end

function InviteOMatic:IsDebugEnabled(info)
	return self.db.profile.debug
end

function InviteOMatic:ToggleDebugEnabled(info,newEnabledValue)
	self.db.profile.debug = not self.db.profile.debug
end

function InviteOMatic:ToggleConvertTriggeringEnabled(info,newEnabledValue)
	self.db.profile.raidEnable = not self.db.profile.raidEnable
end

function InviteOMatic:IsConvertTriggeringEnabled(info)
	return self.db.profile.raidEnable
end

function InviteOMatic:ToggleTriggeringEnabled(info, newEnabledValue)
	self.db.profile.enabled = not self.db.profile.enabled
end

function InviteOMatic:IsReportDeclineEnabled(info)
	return self.db.profile.reportDecline
end

function InviteOMatic:ToggleReportDeclineEnabled(info, newValue)
	self.db.profile.reportDecline = not self.db.profile.reportDecline
end

function InviteOMatic:IsAnnounceEnabled(info)
	return self.db.profile.announce
end

function InviteOMatic:ToggleAnnounceEnabled(info,newEnabledValue)
	self.db.profile.announce = not self.db.profile.announce
end
